/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Mobility Components.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef S60CAMERACONTROL_H
#define S60CAMERACONTROL_H

#include <qcameracontrol.h>
#include <QtCore/qobject.h>

#include "s60cameraengineobserver.h"    // MCameraEngineObserver
#include "s60videocapturesession.h"     // TVideoCaptureState

#include <e32base.h>
#include <fbs.h>

QT_USE_NAMESPACE

class S60CameraService;
class S60ImageCaptureSession;
class S60VideoCaptureSession;
class CCameraEngine;
class CCameraViewfinderEngine;

class S60CameraControl : public QCameraControl, public MCameraEngineObserver
{
    Q_OBJECT
    
public: // Enums
    
    enum Error {
        NoError = 0,
        OutOfMemoryError,
        InUseError,
        NotReadyError,
        UnknownError = -1
    };
    
    enum EcamErrors {
        KErrECamCameraDisabled = -12100,        // The camera has been disabled, hence calls do not succeed
        KErrECamSettingDisabled = -12101,       // This parameter or operation is supported, but presently is disabled
        KErrECamParameterNotInRange = -12102,   // This value is out of range
        KErrECamSettingNotSupported = -12103,   // This parameter or operation is not supported
        KErrECamNotOptimalFocus = -12104        // The optimum focus is lost
    };
    
    enum ViewfinderOutputType {
        VideoWidgetOutput,
        VideoRendererOutput,
        VideoWindowOutput
    };
    
public: // Constructors & Destructor
    
    S60CameraControl(QObject *parent = 0);
    S60CameraControl(S60VideoCaptureSession *videosession, 
                     S60ImageCaptureSession *imagesession, 
                     QObject *parent = 0);
    ~S60CameraControl();
    
public: // QCameraControl
    
    // State
    QCamera::State state() const;
    void setState(QCamera::State state);

    // Status
    QCamera::Status status() const;
    
    // Capture Mode
    QCamera::CaptureMode captureMode() const;
    void setCaptureMode(QCamera::CaptureMode);
    bool isCaptureModeSupported(QCamera::CaptureMode mode) const;
    
    // Property Setting
    bool canChangeProperty(QCameraControl::PropertyChangeType changeType, QCamera::Status status) const;

    /*
Q_SIGNALS:
    void stateChanged(QCamera::State);
    void statusChanged(QCamera::Status);
    void error(int error, const QString &errorString);
    void captureModeChanged(QCamera::CaptureMode);
*/

public: // Internal
    
    void setError(TInt aError);
    
    // To provide QVideoDeviceControl info
    static int deviceCount();
    static QString name(const int index);
    static QString description(const int index);
    int defaultDevice() const;
    int selectedDevice() const;
    void setSelectedDevice(int index);

    void setVideoOutput(QObject *output, ViewfinderOutputType type);
    
public Q_SLOTS: // Internal slots

    void videoStateChanged(const S60VideoCaptureSession::TVideoCaptureState state);
    
protected: // MCameraEngineObserver
    
    void MceoCameraReady();
    void MceoFocusComplete();
    void MceoCapturedDataReady(TDesC8* aData);
    void MceoCapturedBitmapReady(CFbsBitmap* aBitmap);
    void MceoHandleError(TCameraEngineError aErrorType, TInt aError);
    
private: // Internal Helper Operations
    
    QCamera::Error fromSymbianErrorToQtMultimediaError(int aError);
    
    void loadCamera();
    void unloadCamera();
    void startCamera();
    void stopCamera();
    
    void resetCamera();
    void setCameraHandles();
    void releaseAllResources();

Q_SIGNALS: // Internal Signals

    void cameraReadyChanged(bool);

private: // Data
    
    CCameraEngine           *m_cameraEngine;
    CCameraViewfinderEngine *m_viewfinderEngine;
    S60ImageCaptureSession  *m_imageSession;
    S60VideoCaptureSession  *m_videoSession;
    QObject                 *m_videoOutput;
    QCamera::CaptureMode    m_captureMode;
    QCamera::Status         m_internalState;
    QCamera::State          m_requestedState;
    int                     m_deviceIndex;
    mutable int             m_error;
    bool                    m_startWhenLoaded;
};

#endif // S60CAMERACONTROL_H
