/****************************************************************************
 **
 ** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the Qt Mobility Components.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Solutions Commercial License Agreement provided
 ** with the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 2.1 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.LGPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
 ** rights.  These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
 ** General Public License version 3.0 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** Please note Third Party Software included with Qt Solutions may impose
 ** additional restrictions and it is the user's responsibility to ensure
 ** that they have met the licensing requirements of the GPL, LGPL, or Qt
 ** Solutions Commercial license and the relevant license of the Third
 ** Party Software they are using.
 **
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/

//Backend
#include "qmdegalleryitemresultset.h"
#include "qgallerymdsutility.h"
#include "qmdesession.h"
//API
#include "qdocumentgallery.h"
#include "qgalleryitemrequest.h"

QTM_BEGIN_NAMESPACE

QMDEGalleryItemResultSet::QMDEGalleryItemResultSet(QMdeSession *session, QObject *parent)
:QMDEGalleryResultSet(parent)
{
    m_request = static_cast<QGalleryItemRequest *>(parent);
    m_live = m_request->autoUpdate();
    m_session = session;
    m_resultObject = NULL;

    createQuery();
}

QMDEGalleryItemResultSet::~QMDEGalleryItemResultSet()
{
    delete m_resultObject;

    m_session->RemoveObjectObserver( *this );
}

void QMDEGalleryItemResultSet::HandleObjectNotification( CMdESession& /*aSession*/,
    TObserverNotificationType aType,
    const RArray<TItemId>& aObjectIdArray )
{
    if (aType == ENotifyModify) {
        delete m_resultObject;
        TRAP_IGNORE( m_resultObject = m_session->GetFullObjectL( aObjectIdArray[0] ) );
        if (m_resultObject) {
            QString type = QDocumentGalleryMDSUtility::GetItemTypeFromMDEObject( m_resultObject );
            QStringList propertyList;
            QDocumentGalleryMDSUtility::GetDataFieldsForItemType( propertyList, type );
            QList<int> keys;
            const int propertyCount = propertyList.count();
            for (int i = 0; i < propertyCount; i++) {
                keys.append( QDocumentGalleryMDSUtility::GetPropertyKey( propertyList[i] ));
            }
            if (currentIndex() == 0) {
                emit currentItemChanged();
            }
            emit metaDataChanged( 0, 1, keys );
        }
    } else if (aType == ENotifyRemove) {
        delete m_resultObject;
        m_resultObject = NULL;
        m_isValid = false;
        emit itemsRemoved(0, 1);
        if (currentIndex() == 0) {
            emit currentItemChanged();
        }
    }
}

void QMDEGalleryItemResultSet::createQuery()
{
    TRAP_IGNORE( m_resultObject = m_session->GetFullObjectL( m_request->itemId().toUInt() ) );
    // After that resultObject contains NULL or the needed item
    if (m_resultObject) {
        m_currentObjectIDs.Append( m_resultObject->Id() );
        if (m_live) {
            TRAPD( err,
                m_session->AddItemChangedObserverL( *this, m_currentObjectIDs );
            );
            if (err) {
                m_live = false;
            }
        }
        finish(m_live);
    } else {
        error(QDocumentGallery::ConnectionError);
    }
}

#include "moc_qmdegalleryitemresultset.cpp"
QTM_END_NAMESPACE
